@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/bozo/bosserver.c,v 1.32.2.9 2008/02/04 03:51:41 jaltman Exp $");
 
 #include <afs/stds.h>
 #include <sys/types.h>
@@ -40,18 +38,19 @@ RCSID
 #include "bnode.h"
 #include "bosprototypes.h"
 #include <rx/rxkad.h>
+#include <rx/rxstat.h>
 #include <afs/keys.h>
 #include <afs/ktime.h>
 #include <afs/afsutil.h>
 #include <afs/fileutil.h>
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
+#include <afs/audit.h>
+#include <afs/cellconfig.h>
 #if defined(AFS_SGI_ENV)
 #include <afs/afs_args.h>
 #endif
 
 #define BOZO_LWP_STACKSIZE	16000
-extern int BOZO_ExecuteRequest();
-extern int RXSTATS_ExecuteRequest();
 extern struct bnode_ops fsbnode_ops, dafsbnode_ops, ezbnode_ops, cronbnode_ops;
 
 struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
@@ -102,7 +101,7 @@ bozo_rxstat_userok(struct rx_call *call)
 
 /* restart bozo process */
 int
-bozo_ReBozo()
+bozo_ReBozo(void)
 {
 #ifdef AFS_NT40_ENV
     /* exit with restart code; SCM integrator process will restart bosserver */
@@ -187,7 +186,7 @@ MakeDir(const char *adir)
 
 /* create all the bozo dirs */
 static int
-CreateDirs()
+CreateDirs(void)
 {
     if ((!strncmp
 	 (AFSDIR_USR_DIRPATH, AFSDIR_CLIENT_ETC_DIRPATH,
@@ -239,9 +238,10 @@ StripLine(register char *abuffer)
 }
 
 /* write one bnode's worth of entry into the file */
-static
-bzwrite(register struct bnode *abnode, register struct bztemp *at)
+static int
+bzwrite(register struct bnode *abnode, void *arock)
 {
+    register struct bztemp *at = (struct bztemp *)arock;
     register int i;
     char tbuffer[BOZO_BSSIZE];
     register afs_int32 code;
@@ -293,7 +293,6 @@ ReadBozoFile(char *aname)
 	    if (code < 0)
 		perror("bosconfig rename");
 	}
-#ifdef BOS_NEW_CONFIG
 	if (access(AFSDIR_SERVER_BOZCONFNEW_FILEPATH, 0) == 0) {
 	    code =
 		renamefile(AFSDIR_SERVER_BOZCONFNEW_FILEPATH,
@@ -301,7 +300,6 @@ ReadBozoFile(char *aname)
 	    if (code < 0)
 		perror("bosconfig rename");
 	}
-#endif
     }
 
     /* setup default times we want to do restarts */
@@ -494,7 +492,7 @@ WriteBozoFile(char *aname)
 }
 
 static int
-bdrestart(register struct bnode *abnode, char *arock)
+bdrestart(register struct bnode *abnode, void *arock)
 {
     register afs_int32 code;
 
@@ -514,8 +512,8 @@ bdrestart(register struct bnode *abnode, char *arock)
 
 #define	BOZO_MINSKIP 3600	/* minimum to advance clock */
 /* lwp to handle system restarts */
-static int
-BozoDaemon()
+static void *
+BozoDaemon(void *unused)
 {
     register afs_int32 now;
 
@@ -550,11 +548,12 @@ BozoDaemon()
 	    bnode_ApplyInstance(bdrestart, 0);
 	}
     }
+    return NULL;
 }
 
 #ifdef AFS_AIX32_ENV
 static int
-tweak_config()
+tweak_config(void)
 {
     FILE *f;
     char c[80];
@@ -724,6 +723,7 @@ main(int argc, char **argv, char **envp)
     char namebuf[AFSDIR_PATH_MAX];
     int rxMaxMTU = -1;
     afs_uint32 host = htonl(INADDR_ANY);
+    char *auditFileName = NULL;
 #ifndef AFS_NT40_ENV
     int nofork = 0;
     struct stat sb;
@@ -824,49 +824,30 @@ main(int argc, char **argv, char **envp)
 	else if (strcmp(argv[code], "-allow-dotted-principals") == 0) {
 	    rxkadDisableDotCheck = 1;
 	}
-	else if (!strcmp(argv[i], "-rxmaxmtu")) {
-	    if ((i + 1) >= argc) {
+	else if (!strcmp(argv[code], "-rxmaxmtu")) {
+	    if ((code + 1) >= argc) {
 		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
 		exit(1); 
 	    }
-	    rxMaxMTU = atoi(argv[++i]);
+	    rxMaxMTU = atoi(argv[++code]);
 	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
-		printf("rxMaxMTU %d invalid; must be between %d-%d\n",
+		printf("rxMaxMTU %d invalid; must be between %d-%lu\n",
 			rxMaxMTU, RX_MIN_PACKET_SIZE, 
 			RX_MAX_PACKET_DATA_SIZE);
 		exit(1);
 	    }
 	}
 	else if (strcmp(argv[code], "-auditlog") == 0) {
-	    int tempfd, flags;
-	    FILE *auditout;
-	    char oldName[MAXPATHLEN];
-	    char *fileName = argv[++code];
+	    auditFileName = argv[++code];
 
-#ifndef AFS_NT40_ENV
-	    struct stat statbuf;
-	    
-	    if ((lstat(fileName, &statbuf) == 0) 
-		&& (S_ISFIFO(statbuf.st_mode))) {
-		flags = O_WRONLY | O_NONBLOCK;
-	    } else 
-#endif
-	    {
-		strcpy(oldName, fileName);
-		strcat(oldName, ".old");
-		renamefile(fileName, oldName);
-		flags = O_WRONLY | O_TRUNC | O_CREAT;
+	} else if (strcmp(argv[code], "-audit-interface") == 0) {
+	    char *interface = argv[++code];
+
+	    if (osi_audit_interface(interface)) {
+		printf("Invalid audit interface '%s'\n", interface);
+		exit(1);
 	    }
-	    tempfd = open(fileName, flags, 0666);
-	    if (tempfd > -1) {
-		auditout = fdopen(tempfd, "a");
-		if (auditout) {
-		    osi_audit_file(auditout);
-		} else
-		    printf("Warning: auditlog %s not writable, ignored.\n", fileName);
-	    } else
-		printf("Warning: auditlog %s not writable, ignored.\n", fileName);
 	}
 	else {
 
@@ -875,6 +856,7 @@ main(int argc, char **argv, char **envp)
 #ifndef AFS_NT40_ENV
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
+		   "[-audit-interafce <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
@@ -882,6 +864,7 @@ main(int argc, char **argv, char **envp)
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
+		   "[-audit-interafce <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-enable_peer_stats] [-enable_process_stats] "
 		   "[-help]\n");
@@ -891,6 +874,9 @@ main(int argc, char **argv, char **envp)
 	    exit(0);
 	}
     }
+    if (auditFileName) {
+	osi_audit_file(auditFileName);
+    }
 
 #ifndef AFS_NT40_ENV
     if (geteuid() != 0) {
@@ -970,7 +956,7 @@ main(int argc, char **argv, char **envp)
     }
 
     code = LWP_CreateProcess(BozoDaemon, BOZO_LWP_STACKSIZE, /* priority */ 1,
-			     (void *) /*parm */ 0, "bozo-the-clown",
+			     /* param */ NULL , "bozo-the-clown",
 			     &bozo_pid);
 
     /* try to read the key from the config file */
@@ -1009,7 +995,7 @@ main(int argc, char **argv, char **envp)
     }
 
     /* read init file, starting up programs */
-    if (code = ReadBozoFile(0)) {
+    if ((code = ReadBozoFile(0))) {
 	bozo_Log
 	    ("bosserver: Something is wrong (%d) with the bos configuration file %s; aborting\n",
 	     code, AFSDIR_SERVER_BOZCONF_FILEPATH);
@@ -1040,7 +1026,6 @@ main(int argc, char **argv, char **envp)
 
     if (rxBind) {
 	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
         if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
@@ -1049,7 +1034,6 @@ main(int argc, char **argv, char **envp)
                                            AFSDIR_SERVER_NETINFO_FILEPATH,
                                            AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else 
-#endif	
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
@@ -1067,8 +1051,7 @@ main(int argc, char **argv, char **envp)
     rx_SetStackSize(tservice, BOZO_LWP_STACKSIZE);	/* so gethostbyname works (in cell stuff) */
     if (rxkadDisableDotCheck) {
         rx_SetSecurityConfiguration(tservice, RXS_CONFIG_FLAGS,
-                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK, 
-                                    NULL);
+                                    (void *)RXS_CONFIG_FLAGS_DISABLE_DOTCHECK);
     }
 
     tservice =
