@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/butc/tcmain.c,v 1.16.8.6 2007/10/31 04:09:25 shadow Exp $");
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -53,9 +51,12 @@ RCSID
 #include <ctype.h>
 #include "error_macros.h"
 #include <afs/budb_errs.h>
+#include <afs/budb_client.h>
+#include <afs/bucoord_prototypes.h>
 #include "afs/butx.h"
 #define XBSA_TCMAIN
 #include "butc_xbsa.h"
+#include "butc_prototypes.h"
 
 #define N_SECURITY_OBJECTS 3
 #define ERRCODE_RANGE 8		/* from error_table.h */
@@ -65,8 +66,6 @@ RCSID
 #define CFG_PREFIX "CFG"
 
 struct ubik_client *cstruct;
-extern void TC_ExecuteRequest();
-extern int dbWatcher();
 FILE *logIO, *ErrorlogIO, *centralLogIO, *lastLogIO;
 char lFile[AFSDIR_PATH_MAX];
 char logFile[256];
@@ -112,7 +111,8 @@ afs_uint32 SHostAddrs[ADDRSPERSITE];
 
 /* dummy routine for the audit work.  It should do nothing since audits */
 /* occur at the server level and bos is not a server. */
-osi_audit()
+int
+osi_audit(void)
 {
     return 0;
 }
@@ -124,7 +124,7 @@ SafeATOL(register char *anum)
     register int tc;
 
     total = 0;
-    while (tc = *anum) {
+    while ((tc = *anum)) {
 	if (tc < '0' || tc > '9')
 	    return -1;
 	total *= 10;
@@ -250,7 +250,8 @@ atocl(char *numstring, char crunit, afs_int32 *number)
 }
 
 /* replace last two ocurrences of / by _ */
-static
+#if 0
+static int
 stringReplace(char *name)
 {
     char *pos;
@@ -264,8 +265,9 @@ stringReplace(char *name)
     strcat(name, buffer);
     return 0;
 }
+#endif
 
-static
+static int
 stringNowReplace(char *logFile, char *deviceName)
 {
     char *pos = 0;
@@ -283,7 +285,7 @@ stringNowReplace(char *logFile, char *deviceName)
 	deviceName += devPrefLen;
 	mvFlag++;
     }
-    while (pos = strchr(deviceName, devPrefix[0]))	/* look for / or \ */
+    while ((pos = strchr(deviceName, devPrefix[0])))	/* look for / or \ */
 	*pos = '_';
     strcat(logFile, deviceName);
     /* now put back deviceName to the way it was */
@@ -796,7 +798,7 @@ GetConfigParams(char *filename, afs_int32 port)
 	statusSize *= BufferSize;
 	if (statusSize < 0)
 	    statusSize = 0x7fffffff;	/*max size */
-	printf("Status every %ld Bytes\n", statusSize);
+	printf("Status every %ld Bytes\n", afs_printable_int32_ld(statusSize));
     }
 
   error_exit:
@@ -1034,7 +1036,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
 
     if (rxBind) {
         afs_int32 ccode;
-#ifndef AFS_NT40_ENV
         if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
@@ -1043,7 +1044,6 @@ WorkerBee(struct cmd_syndesc *as, void *arock)
                                            AFSDIR_SERVER_NETINFO_FILEPATH,
                                            AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else 
-#endif	
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
