@@ -49,10 +49,11 @@
 #include "update.h"
 #include "global.h"
 
-extern int UPDATE_ExecuteRequest();
-
 static int AddObject(char **expPath, char *dir);
 static int PathInDirectory(char *dir, char *path);
+int update_SendFile(int, struct rx_call *, struct stat *);
+int update_SendDirInfo(char *, struct rx_call *, struct stat *,
+		       char *origDir);
 
 struct afsconf_dir *cdir;
 int nDirs;
@@ -60,7 +61,7 @@ char *dirName[MAXENTRIES];
 int dirLevel[MAXENTRIES];
 char *whoami;
 
-static int Quit();
+static int Quit(char *);
 
 int rxBind = 0;
 
@@ -165,7 +166,7 @@ AuthOkay(struct rx_call *call, char *name)
 }
 
 int
-osi_audit()
+osi_audit(void)
 {
 /* this sucks but it works for now.
 */
@@ -276,7 +277,6 @@ main(int argc, char *argv[])
 
     if (rxBind) {
 	afs_int32 ccode;
-#ifndef AFS_NT40_ENV
         if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
             AFSDIR_SERVER_NETINFO_FILEPATH) {
             char reason[1024];
@@ -285,7 +285,6 @@ main(int argc, char *argv[])
                                            AFSDIR_SERVER_NETINFO_FILEPATH,
                                            AFSDIR_SERVER_NETRESTRICT_FILEPATH);
         } else 
-#endif	
 	{
             ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
         }
@@ -314,10 +313,7 @@ main(int argc, char *argv[])
 	rxkad_NewServerSecurityObject(rxkad_clear, cdir, afsconf_GetKey, 0);
     if (securityObjects[2] == (struct rx_securityClass *)0)
 	Quit("rxkad_NewServerSecurityObject");
-#ifdef USE_RXKAD_KEYTAB
-    if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
-        rxkad_BindKeytabDecrypt(securityObjects[2]);
-#endif
+
     /* Instantiate a single UPDATE service.  The rxgen-generated procedure
      * which is called to decode requests is passed in here
      * (UPDATE_ExecuteRequest). */
@@ -405,10 +401,9 @@ UPDATE_FetchInfo(struct rx_call *call, char *name)
 }
 
 static int
-Quit(msg, a, b)
-     char *msg;
+Quit(char *msg)
 {
-    fprintf(stderr, msg, a, b);
+    fprintf(stderr, msg);
     exit(1);
 }
 
