@@ -12,8 +12,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/util/winsock_nt.c,v 1.6.4.1 2006/07/03 01:06:45 jaltman Exp $");
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
@@ -25,18 +23,13 @@ RCSID
  * 
  * Returns 0 on success, -1 on error.
  */
-static int once = 1;
-
 int
 afs_winsockInit(void)
 {
-    if (once) {
 	int code;
 	WSADATA data;
 	WORD sockVersion;
 
-	once = 0;
-
 	sockVersion = 2;
 	code = WSAStartup(sockVersion, &data);
 	if (code)
@@ -44,7 +37,6 @@ afs_winsockInit(void)
 
 	if (data.wVersion != 2)
 	    return -1;
-    }
     return 0;
 }
 
@@ -52,7 +44,6 @@ void
 afs_winsockCleanup(void)
 {
     WSACleanup();
-    once = 0;
 }
 
 /* This function will begin to fail in the year 2038 */
